@@ -85,7 +85,6 @@ class GH_Mixture_Model
         
         // General Methods
         void random_soft_init(void); // self explanitory  
-        double calculate_log_liklihood(void); // calcuates the logliklihood function over all 
         void M_step_alphas(void); // calsculate alpha_gs under infinite logliklihood problem. 
 
 
@@ -114,17 +113,30 @@ class GH_Mixture_Model
         // stochastic methods
         void SE_step(void); // performs the stochastic estep .
         void RE_step(void);  
+        // semi-supervised learning. 
+        void SEMI_step(void); 
+        arma::vec semi_labels; // for semi-supervised learning. 
+        double calculate_log_liklihood_std(void); 
+        double calculate_log_liklihood_semi(void); 
+        double calculate_log_liklihood(void){
+          return (this->*calculate_log_liklihood_hidden)(); 
+        }
         void (GH_Mixture_Model::*e_step)(void); 
+        double (GH_Mixture_Model::*calculate_log_liklihood_hidden)(void); 
 
-        void set_E_step(bool stochastic){
-            if(stochastic){
+        void set_E_step(int stochastic){
+            if(stochastic == 1){
                 this->e_step = &GH_Mixture_Model::SE_step; 
             }
+            if(stochastic == 2){
+                this->e_step = &GH_Mixture_Model::SEMI_step; 
+                this->calculate_log_liklihood_hidden = &GH_Mixture_Model::calculate_log_liklihood_semi; 
+            }
             else{
                 // e_step is already regular. 
             }
         };
-        
+
         void M_step_props(); // calculate the proportions and n_gs
         void M_step_init_gaussian(void); // initializes mu, sig, and alphas according to gaussian settings. alpha is really small.  
         double k_bessel(double nu, double x); 
@@ -314,6 +326,8 @@ GH_Mixture_Model::GH_Mixture_Model(arma::mat* in_data, int in_G, int model_id ){
     nu_d = 1.0; 
     EYE = arma::eye(p,p); 
     e_step = &GH_Mixture_Model::RE_step; 
+    semi_labels = arma::vec(n,arma::fill::zeros); 
+    calculate_log_liklihood_hidden = &GH_Mixture_Model::calculate_log_liklihood_std; 
 }
 
 
@@ -534,7 +548,7 @@ double GH_Mixture_Model::log_density(arma::vec x, // vector comes in as 1 x p
 
 
 
-double GH_Mixture_Model::calculate_log_liklihood(void) {
+double GH_Mixture_Model::calculate_log_liklihood_std(void) {
 
   double lglik = 0.0;
   double row_sum_term = 0.0; 
@@ -559,6 +573,47 @@ double GH_Mixture_Model::calculate_log_liklihood(void) {
 
 
 
+double GH_Mixture_Model::calculate_log_liklihood_semi(void) {
+
+  double lglik = 0.0;
+  double row_sum_term = 0.0; 
+  // go through observations.
+  for(int i = 0; i < n; i++)
+  {
+
+    if(semi_labels.at(i) == 0) { 
+      // go through groups
+      row_sum_term = 0.0; 
+      for(int g = 0; g < G; g++) 
+      {
+        row_sum_term += pi_gs[g]*exp(log_density(data.col(i),mus[g],alphas[g], // parameters and observations
+                          a_is[g].at(i),c_is[g].at(i),b_is[g].at(i), // latent variables
+                          inv_sigs[g],log_dets[g],omegas[g],lambdas[g] // other parameters 
+                          )); 
+      } 
+      row_sum_term = std::log(row_sum_term); 
+      lglik += row_sum_term;
+    }
+    else {
+
+      row_sum_term = 0.0; 
+      for(int g = 0; g < G; g++){
+        row_sum_term += zi_gs.at(i,g)*( log(pi_gs.at(g)) +  log_density(data.col(i),mus[g],alphas[g], // parameters and observations
+                          a_is[g].at(i),c_is[g].at(i),b_is[g].at(i), // latent variables
+                          inv_sigs[g],log_dets[g],omegas[g],lambdas[g] // other parameters 
+                          )); 
+      }
+
+      lglik += row_sum_term; 
+
+    }
+
+  }
+
+  return lglik; 
+  
+}
+
 void GH_Mixture_Model::M_step_props() {
 
   // initialize mixing proportions and n_gs 
@@ -805,6 +860,87 @@ void GH_Mixture_Model::RE_step()
 }
 
 
+#pragma once
+void GH_Mixture_Model::SEMI_step(void)
+{
+  
+  // set up inter_mediate step for z_igs. 
+  arma::mat inter_zigs = arma::mat(n,G,arma::fill::zeros); 
+
+  // intermediate values 
+  arma::rowvec inter_density = arma::rowvec(G,arma::fill::zeros); 
+  double inter_row_sum; 
+
+  // calculate density proportions 
+  for(int i = 0; i < n; i++) 
+  { 
+
+    if( semi_labels.at(i) == 0) {
+      // clear row sum for every observation 
+      inter_row_sum = 0;
+      inter_density = arma::rowvec(G,arma::fill::zeros); 
+
+      for(int g = 0; g < G; g ++)
+      {
+        // numerator in e step term for mixture models
+        
+        inter_density[g] = std::pow((pi_gs[g])*std::exp(log_density(data.col(i),mus[g],alphas[g], // basic parameters
+                                                        a_is[g][i],c_is[g][i],b_is[g][i], // latent parameters
+                                                        inv_sigs[g],log_dets[g],omegas[g],lambdas[g])),nu);
+        inter_row_sum += inter_density[g]; 
+      }
+        // after calculating inter_density assign the row to the z_ig matrix. 
+      for(int g = 0; g < G; g ++)
+      {
+        
+        double numer_g = inter_row_sum - inter_density[g]; 
+        double denom_g = inter_density[g]; 
+
+        inter_zigs.at(i,g) = 1.0/(1 + numer_g/denom_g);
+
+      }
+
+      double ss = arma::sum(inter_zigs.row(i));
+      
+      if(isnan(ss)){
+        inter_zigs.row(i) = zi_gs.row(i);
+        ss = arma::sum(inter_zigs.row(i));
+      }
+
+      // make sure that it adds up to one
+      int count = 0;
+      while(true) {
+
+        if(comparison_gp(ss,1.0)){ 
+          break; 
+        } 
+        double push_sum = 0.0; 
+
+        for(int gv = 0; gv < (G-1); gv++){
+          push_sum += inter_zigs.row(i)[gv]; 
+        }
+        
+        inter_zigs.row(i)[G-1] = 1.0 - push_sum; 
+        ss = inter_zigs.row(i)[G-1] + push_sum;
+
+        if(count == 10){
+          
+          inter_zigs.row(i) = zi_gs.row(i); // reset to last a posterori
+          // loggy(zi_gs.row(i) space  i );
+          break; 
+        }
+        count ++ ;
+      }
+    }
+    else {
+      inter_zigs.at(i,semi_labels.at(i) - 1) = 1; 
+    }
+  }
+
+  zi_gs = inter_zigs; 
+}
+
+
 
 // LATENT E STEP 
 void GH_Mixture_Model::E_step_latent(void)
@@ -1054,6 +1190,8 @@ void GH_Mixture_Model::EM_burn(int in_burn_steps)
   // copy dataset and z_igs. 
   arma::mat* orig_data = new arma::mat(p,n); // create empty arma mat on the heap. 
   arma::mat* orig_zi_gs = new arma::mat(n,G); 
+  arma::vec* orig_semi_labels = new arma::vec(n,arma::fill::zeros);  
+
 
   std::vector < arma::vec  > orig_a_is = a_is; 
   std::vector < arma::vec  > orig_b_is = b_is; 
@@ -1062,10 +1200,14 @@ void GH_Mixture_Model::EM_burn(int in_burn_steps)
 
   *orig_data = data; // set orig_data. 
   *orig_zi_gs = zi_gs; // set zi_igs. 
+  *orig_semi_labels = semi_labels; 
+
 
   // remove all data, and zi_gs with missing values. 
   data.shed_cols(col_tags); 
   zi_gs.shed_rows(col_tags); 
+  semi_labels.shed_rows(col_tags); 
+
 
   for(int g = 0; g < G; g++ )
   {
